@@ -1,10 +1,20 @@
 from __future__ import annotations
 
+import datetime
 import os
 import sys
 
 from pathlib import Path
-from PySide6.QtCore import QDate, QTimer, Qt, QSettings, Slot, QUrl, QEvent
+from PySide6.QtCore import (
+    QDate,
+    QTimer,
+    Qt,
+    QSettings,
+    Slot,
+    QUrl,
+    QEvent,
+    QSignalBlocker,
+)
 from PySide6.QtGui import (
     QAction,
     QCursor,
@@ -12,6 +22,7 @@ from PySide6.QtGui import (
     QFont,
     QGuiApplication,
     QTextCharFormat,
+    QTextListFormat,
 )
 from PySide6.QtWidgets import (
     QCalendarWidget,
@@ -94,8 +105,8 @@ class _LockOverlay(QWidget):
 
 
 class MainWindow(QMainWindow):
-    def __init__(self):
-        super().__init__()
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
         self.setWindowTitle(APP_NAME)
         self.setMinimumSize(1000, 650)
 
@@ -149,6 +160,10 @@ class MainWindow(QMainWindow):
         self.toolBar.numbersRequested.connect(self.editor.toggle_numbers)
         self.toolBar.alignRequested.connect(self.editor.setAlignment)
         self.toolBar.historyRequested.connect(self._open_history)
+        self.toolBar.insertImageRequested.connect(self._on_insert_image)
+
+        self.editor.currentCharFormatChanged.connect(lambda _f: self._sync_toolbar())
+        self.editor.cursorPositionChanged.connect(self._sync_toolbar)
 
         split = QSplitter()
         split.addWidget(left_panel)
@@ -201,6 +216,10 @@ class MainWindow(QMainWindow):
         act_export.setShortcut("Ctrl+E")
         act_export.triggered.connect(self._export)
         file_menu.addAction(act_export)
+        act_backup = QAction("&Backup", self)
+        act_backup.setShortcut("Ctrl+Shift+B")
+        act_backup.triggered.connect(self._backup)
+        file_menu.addAction(act_backup)
         file_menu.addSeparator()
         act_quit = QAction("&Quit", self)
         act_quit.setShortcut("Ctrl+Q")
@@ -210,21 +229,21 @@ class MainWindow(QMainWindow):
         # Navigate menu with next/previous/today
         nav_menu = mb.addMenu("&Navigate")
         act_prev = QAction("Previous Day", self)
-        act_prev.setShortcut("Ctrl+<")
+        act_prev.setShortcut("Ctrl+Shift+P")
         act_prev.setShortcutContext(Qt.ApplicationShortcut)
         act_prev.triggered.connect(lambda: self._adjust_day(-1))
         nav_menu.addAction(act_prev)
         self.addAction(act_prev)
 
         act_next = QAction("Next Day", self)
-        act_next.setShortcut("Ctrl+>")
+        act_next.setShortcut("Ctrl+Shift+N")
         act_next.setShortcutContext(Qt.ApplicationShortcut)
         act_next.triggered.connect(lambda: self._adjust_day(1))
         nav_menu.addAction(act_next)
         self.addAction(act_next)
 
         act_today = QAction("Today", self)
-        act_today.setShortcut("Ctrl+T")
+        act_today.setShortcut("Ctrl+Shift+T")
         act_today.setShortcutContext(Qt.ApplicationShortcut)
         act_today.triggered.connect(self._adjust_today)
         nav_menu.addAction(act_today)
@@ -315,6 +334,61 @@ class MainWindow(QMainWindow):
             pass
 
     # --- UI handlers ---------------------------------------------------------
+
+    def _sync_toolbar(self):
+        fmt = self.editor.currentCharFormat()
+        c = self.editor.textCursor()
+        bf = c.blockFormat()
+
+        # Block signals so setChecked() doesn't re-trigger actions
+        blocker1 = QSignalBlocker(self.toolBar.actBold)
+        blocker2 = QSignalBlocker(self.toolBar.actItalic)
+        blocker3 = QSignalBlocker(self.toolBar.actUnderline)
+        blocker4 = QSignalBlocker(self.toolBar.actStrike)
+
+        self.toolBar.actBold.setChecked(fmt.fontWeight() == QFont.Weight.Bold)
+        self.toolBar.actItalic.setChecked(fmt.fontItalic())
+        self.toolBar.actUnderline.setChecked(fmt.fontUnderline())
+        self.toolBar.actStrike.setChecked(fmt.fontStrikeOut())
+
+        # Headings: decide which to check by current point size
+        def _approx(a, b, eps=0.5):  # small float tolerance
+            return abs(float(a) - float(b)) <= eps
+
+        cur_size = fmt.fontPointSize() or self.editor.font().pointSizeF()
+
+        bH1 = _approx(cur_size, 24)
+        bH2 = _approx(cur_size, 18)
+        bH3 = _approx(cur_size, 14)
+
+        b1 = QSignalBlocker(self.toolBar.actH1)
+        b2 = QSignalBlocker(self.toolBar.actH2)
+        b3 = QSignalBlocker(self.toolBar.actH3)
+        bN = QSignalBlocker(self.toolBar.actNormal)
+
+        self.toolBar.actH1.setChecked(bH1)
+        self.toolBar.actH2.setChecked(bH2)
+        self.toolBar.actH3.setChecked(bH3)
+        self.toolBar.actNormal.setChecked(not (bH1 or bH2 or bH3))
+
+        # Lists
+        lst = c.currentList()
+        bullets_on = lst and lst.format().style() == QTextListFormat.Style.ListDisc
+        numbers_on = lst and lst.format().style() == QTextListFormat.Style.ListDecimal
+        QSignalBlocker(self.toolBar.actBullets)
+        QSignalBlocker(self.toolBar.actNumbers)
+        self.toolBar.actBullets.setChecked(bool(bullets_on))
+        self.toolBar.actNumbers.setChecked(bool(numbers_on))
+
+        # Alignment
+        align = bf.alignment() & Qt.AlignHorizontal_Mask
+        QSignalBlocker(self.toolBar.actAlignL)
+        self.toolBar.actAlignL.setChecked(align == Qt.AlignLeft)
+        QSignalBlocker(self.toolBar.actAlignC)
+        self.toolBar.actAlignC.setChecked(align == Qt.AlignHCenter)
+        QSignalBlocker(self.toolBar.actAlignR)
+        self.toolBar.actAlignR.setChecked(align == Qt.AlignRight)
+
     def _current_date_iso(self) -> str:
         d = self.calendar.selectedDate()
         return f"{d.year():04d}-{d.month():02d}-{d.day():02d}"
@@ -373,7 +447,7 @@ class MainWindow(QMainWindow):
         """
         if not self._dirty and not explicit:
             return
-        text = self.editor.toHtml()
+        text = self.editor.to_html_with_embedded_images()
         try:
             self.db.save_new_version(date_iso, text, note)
         except Exception as e:
@@ -416,6 +490,18 @@ class MainWindow(QMainWindow):
             self._load_selected_date(date_iso)
             self._refresh_calendar_marks()
 
+    def _on_insert_image(self):
+        # Let the user pick one or many images
+        paths, _ = QFileDialog.getOpenFileNames(
+            self,
+            "Insert image(s)",
+            "",
+            "Images (*.png *.jpg *.jpeg *.bmp *.gif *.webp)",
+        )
+        if not paths:
+            return
+        self.editor.insert_images(paths)  # call into the editor
+
     # ----------- Settings handler ------------#
     def _open_settings(self):
         dlg = SettingsDialog(self.cfg, self.db, self)
@@ -484,13 +570,31 @@ class MainWindow(QMainWindow):
     # ----------------- Export handler ----------------- #
     @Slot()
     def _export(self):
-        try:
-            self.export_dialog()
-        except Exception as e:
-            QMessageBox.critical(self, "Export failed", str(e))
+        warning_title = "Unencrypted export"
+        warning_message = """
+Exporting the database will be unencrypted!
 
-    def export_dialog(self) -> None:
-        filters = "Text (*.txt);;" "JSON (*.json);;" "CSV (*.csv);;" "HTML (*.html);;"
+Are you sure you want to continue?
+
+If you want an encrypted backup, choose Backup instead of Export.
+"""
+        dlg = QMessageBox()
+        dlg.setWindowTitle(warning_title)
+        dlg.setText(warning_message)
+        dlg.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
+        dlg.setIcon(QMessageBox.Warning)
+        dlg.show()
+        dlg.adjustSize()
+        if dlg.exec() != QMessageBox.Yes:
+            return False
+
+        filters = (
+            "Text (*.txt);;"
+            "JSON (*.json);;"
+            "CSV (*.csv);;"
+            "HTML (*.html);;"
+            "SQL (*.sql);;"
+        )
 
         start_dir = os.path.join(os.path.expanduser("~"), "Documents")
         filename, selected_filter = QFileDialog.getSaveFileName(
@@ -504,6 +608,7 @@ class MainWindow(QMainWindow):
             "JSON (*.json)": ".json",
             "CSV (*.csv)": ".csv",
             "HTML (*.html)": ".html",
+            "SQL (*.sql)": ".sql",
         }.get(selected_filter, ".txt")
 
         if not Path(filename).suffix:
@@ -518,14 +623,49 @@ class MainWindow(QMainWindow):
             elif selected_filter.startswith("CSV"):
                 self.db.export_csv(entries, filename)
             elif selected_filter.startswith("HTML"):
-                self.bd.export_html(entries, filename)
+                self.db.export_html(entries, filename)
+            elif selected_filter.startswith("SQL"):
+                self.db.export_sql(filename)
             else:
-                self.bd.export_by_extension(entries, filename)
+                self.db.export_by_extension(filename)
 
             QMessageBox.information(self, "Export complete", f"Saved to:\n{filename}")
         except Exception as e:
             QMessageBox.critical(self, "Export failed", str(e))
 
+    # ----------------- Backup handler ----------------- #
+    @Slot()
+    def _backup(self):
+        filters = "SQLCipher (*.db);;"
+
+        now = datetime.datetime.now().strftime("%Y-%m-%d-%H-%M-%S")
+        start_dir = os.path.join(
+            os.path.expanduser("~"), "Documents", f"bouquin_backup_{now}.db"
+        )
+        filename, selected_filter = QFileDialog.getSaveFileName(
+            self, "Backup encrypted notebook", start_dir, filters
+        )
+        if not filename:
+            return  # user cancelled
+
+        default_ext = {
+            "SQLCipher (*.db)": ".db",
+        }.get(selected_filter, ".db")
+
+        if not Path(filename).suffix:
+            filename += default_ext
+
+        try:
+            if selected_filter.startswith("SQL"):
+                self.db.export_sqlcipher(filename)
+                QMessageBox.information(
+                    self, "Backup complete", f"Saved to:\n{filename}"
+                )
+        except Exception as e:
+            QMessageBox.critical(self, "Backup failed", str(e))
+
+    # ----------------- Help handlers ----------------- #
+
     def _open_docs(self):
         url_str = "https://git.mig5.net/mig5/bouquin/wiki/Help"
         url = QUrl.fromUserInput(url_str)
@@ -542,7 +682,7 @@ class MainWindow(QMainWindow):
                 self, "Open Documentation", f"Couldn't open:\n{url.toDisplayString()}"
             )
 
-    # Idle handlers
+    # ----------------- Idle handlers ----------------- #
     def _apply_idle_minutes(self, minutes: int):
         minutes = max(0, int(minutes))
         if not hasattr(self, "_idle_timer"):
@@ -600,13 +740,14 @@ class MainWindow(QMainWindow):
                 tb.setEnabled(True)
             self._idle_timer.start()
 
-    # Close app handler - save window position and database
+    # ----------------- Close handlers ----------------- #
     def closeEvent(self, event):
         try:
             # Save window position
             self.settings.setValue("main/geometry", self.saveGeometry())
             self.settings.setValue("main/windowState", self.saveState())
             self.settings.setValue("main/maximized", self.isMaximized())
+
             # Ensure we save any last pending edits to the db
             self._save_current()
             self.db.close()
